"use strict";
exports.__esModule = true;
exports.injectPositionals = exports.replaceWildcardWithPositionals = exports.normalizeWildcardString = exports.replace = exports.endsWith = exports.startsWith = exports.append = exports.prepend = void 0;
var compose_1 = require("./compose");
function prepend(prefix) {
    return function (source) { return prefix + source; };
}
exports.prepend = prepend;
function append(suffix) {
    return function (source) { return source + suffix; };
}
exports.append = append;
function startsWith(prefix) {
    return function (source) {
        return source.startsWith(prefix);
    };
}
exports.startsWith = startsWith;
function endsWith(suffix) {
    return function (value) {
        return value.endsWith(suffix);
    };
}
exports.endsWith = endsWith;
function replace(source, replacer) {
    return function (value) {
        return value.replace(source, replacer);
    };
}
exports.replace = replace;
function normalizeWildcardString(source) {
    return compose_1.compose(prepend('^'), append('$'), replace(/\*/g, function () { return '(.*)'; }))(source);
}
exports.normalizeWildcardString = normalizeWildcardString;
function replaceWildcardWithPositionals(source) {
    var wildcardCount = 0;
    return source.replace(/\*/g, function () {
        wildcardCount++;
        return "$" + wildcardCount;
    });
}
exports.replaceWildcardWithPositionals = replaceWildcardWithPositionals;
function injectPositionals(source, positionals) {
    return source.replace(/\$(\d)/g, function (_, group) {
        var positionalIndex = parseInt(group, 10) - 1;
        return positionals[positionalIndex];
    });
}
exports.injectPositionals = injectPositionals;
