"use strict";
exports.__esModule = true;
exports.DotaliasWebpackPlugin = void 0;
var path = require("path");
var compose_1 = require("../utils/compose");
var strings_1 = require("../utils/strings");
var DotaliasWebpackPlugin = /** @class */ (function () {
    function DotaliasWebpackPlugin(config) {
        this.config = this.transformConfig(config);
    }
    DotaliasWebpackPlugin.prototype.apply = function (compiler) {
        var _this = this;
        compiler.hooks.normalModuleFactory.tap('DotaliasWebpackPlugin', function (factory) {
            factory.hooks.beforeResolve.tap('DotaliasWebpackPlugin', function (result) {
                var modulePath = _this.findResourceAlias(result.request);
                if (modulePath) {
                    result.request = modulePath;
                }
            });
        });
    };
    DotaliasWebpackPlugin.prototype.transformConfig = function (config) {
        return Object.entries(config).reduce(function (config, _a) {
            var moduleName = _a[0], modulePath = _a[1];
            config[moduleName] = Array.isArray(modulePath)
                ? modulePath.map(function (modulePath) {
                    return path.resolve(process.cwd(), modulePath);
                })
                : path.resolve(process.cwd(), modulePath);
            return config;
        }, {});
    };
    DotaliasWebpackPlugin.prototype.findResourceAlias = function (actualModuleName) {
        var _this = this;
        var resource = Object.entries(this.config).reduce(function (resource, _a) {
            var moduleName = _a[0], modulePath = _a[1];
            if (typeof resource !== 'undefined') {
                return resource;
            }
            // Alias name is a RegExp-like string.
            if (moduleName.includes('*')) {
                var normalizedModuleName = compose_1.compose(strings_1.replace(/\*/g, function () { return '(.*)'; }))(moduleName);
                var moduleNameRegExp = new RegExp(normalizedModuleName);
                var match = actualModuleName.match(moduleNameRegExp);
                if (!match) {
                    return;
                }
                var positionals_1 = match.slice(1);
                var modulePaths = []
                    .concat(modulePath)
                    .map(strings_1.replaceWildcardWithPositionals)
                    .map(function (absoluteModulePath) {
                    return strings_1.injectPositionals(absoluteModulePath, positionals_1);
                });
                return _this.resolveFirstExistingModule(modulePaths);
            }
            // Alias name is a regular string.
            if (actualModuleName === moduleName) {
                return _this.resolveFirstExistingModule([].concat(modulePath));
            }
        }, undefined);
        return resource;
    };
    DotaliasWebpackPlugin.prototype.resolveFirstExistingModule = function (paths) {
        return paths.find(function (modulePath) {
            try {
                require.resolve(modulePath);
                return true;
            }
            catch (error) {
                return false;
            }
        });
    };
    return DotaliasWebpackPlugin;
}());
exports.DotaliasWebpackPlugin = DotaliasWebpackPlugin;
